@@ -98,12 +98,13 @@ static int hamallowed = HAM_NONE;
 #endif
 
     /*
-     *  Console and Frame Buffer
+     *  Console and Fram Buffer
      */
 
 static int ActiveVT = -1;
 static int ConsoleFd = 0;
 static char *FrameBufferName = NULL;
+static int VC = -1;
 static int FrameBufferFD = -1;
 static caddr_t FrameBufferBits = (caddr_t) - 1;
 static size_t FrameBufferSize;
@@ -276,20 +277,46 @@ static void VTRequest(int signo)
 static void OpenFB(void)
 {
   int i, fd, vtno, colors;
+  int devfs = 0;
   struct fb_fix_screeninfo fix;
   struct fb_var_screeninfo var;
-  char vtname[11];
+  char vtname[50];
   struct vt_stat vts;
   struct vt_mode VT;
 
   if (geteuid())
     Die("%s must be suid root\n", ProgramName);
+  
   if ((fd = open("/dev/tty0", O_WRONLY, 0)) < 0)
-    Die("Cannot open /dev/tty0: %s\n", strerror(errno));
-  if (ioctl(fd, VT_OPENQRY, &vtno) < 0 || vtno == -1)
-    Die("Cannot find a free VT\n");
+    {
+      devfs = 1;
+      if ((fd = open("/dev/vc/0", O_WRONLY, 0)) < 0)
+	Die("Cannot open /dev/tty0 or /dev/vc/0: %s\n", strerror(errno));
+    }
+  
+  if (VC == -1)
+    {
+      if (ioctl(fd, VT_OPENQRY, &vtno) < 0 || vtno == -1)
+	Die("Cannot find a free VT\n");
+    }
+  else
+    vtno = VC;
+  
   close(fd);
-  sprintf(vtname, "/dev/tty%d", vtno);  /* /dev/tty1-64 */
+
+  if (devfs == 1)
+    sprintf(vtname, "/dev/vc/%d", vtno);
+  else
+    sprintf(vtname, "/dev/tty%d", vtno);  /* /dev/tty1-64 */
+  
+  if (!FrameBufferName && !(FrameBufferName = getenv("FRAMEBUFFER")))
+    {
+      if (devfs == 1)
+	FrameBufferName = "/dev/fb/0";
+      else
+	FrameBufferName = "/dev/fb0";
+    }
+
   if ((ConsoleFd = open(vtname, O_RDWR | O_NDELAY, 0)) < 0)
     Die("Cannot open %s: %s\n", vtname, strerror(errno));
   /*
@@ -1393,7 +1420,8 @@ static void Usage(void)
       "        --greyscale\n"
       "    -r, --random-dither    : Use random dithering for grayscale\n"
       "    -s, --stdin            : Use standard input instead of files\n"
-      "    -f, --framebuffer=X    : Specify framebuffer device to use\n\n"
+      "    -f, --framebuffer=X    : Specify framebuffer device to use\n"
+      "    -C, --virtual-console  : Specify virtual console to use\n\n"
       "Advanced Options (only relevant for Amiga HAM modes):\n"
       "    -n, --no-histogram     : Use the default palette instead of\n"
       "                             calculating one.\n"
@@ -1433,8 +1461,9 @@ int main(int argc, char *argv[])
     { "grayscale", 'g', 0, NULL, 'g' },
     { "greyscale", 'g', 0, NULL, 'g' },
     { "random-dither", 'r', 0, NULL, 'r' },
-    { "frame-buffer", 'f', POPT_ARG_STRING, FrameBufferName, 0 },
-    { "framebuffer", 'f', POPT_ARG_STRING, FrameBufferName, 0 },
+    { "frame-buffer", 'f', POPT_ARG_STRING, &FrameBufferName, 0 },
+    { "framebuffer", 'f', POPT_ARG_STRING, &FrameBufferName, 0 },
+    { "virtual-console", 'C', POPT_ARG_INT, &VC, 0},
     { "no-histogram", 'n', 0, NULL, 'n' },
     { "no-ham", 'H', 0, NULL, 'H' },
     { "stdin", 's', 0, NULL, 's' },
@@ -1490,9 +1519,6 @@ int main(int argc, char *argv[])
         "For help, use %s --help\n", ProgramName);
   }
 
-  if (!FrameBufferName && !(FrameBufferName = getenv("FRAMEBUFFER")))
-    FrameBufferName = "/dev/fb0";
-
 #ifndef RDB_DEBUG /* RDB Not needed for debugging */
   signal(SIGSEGV, SigHandler);
   signal(SIGILL, SigHandler);
